#include "Cora.h"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

int read_spec (char *path, int s_flag)
{
  FILE * fd;
  char line[BUFSIZ];
  int count_line = 0, items = 0, lines=0, ibgs=0, i;
  float *berr;

  /*
  int lines=0, theline=0;
  char * ptr;
  double * pwave, * psp, * perr, * pbkgd, * pbkerr, *pbg0;
  */


  if (NULL == (fd = fopen(path, "r")))
    {
      g_warning("%s: File not found.", path);
      return (-1);
    }

  while (fgets(line, BUFSIZ-1, fd))
    if (line[0] != '#') ++count;

  rewind(fd);

  wave = (float *) malloc (sizeof(float) * count);
  flux = (float *) malloc (sizeof(float) * count);
  bb   = (float *) malloc (sizeof(float) * count);
  err  = (float *) malloc (sizeof(float) * count);
  berr = (float *) malloc (sizeof(float) * count);

  if (wave == NULL || flux == NULL || bb == NULL || err == NULL)
    {
      g_warning("Out of memory");
      return(-1);
    }

  while (fgets(line, BUFSIZ-1, fd))
    {
      if (line[0] != '#')
        {
          items = sscanf(line, "%f %f %f %f %f",
			 &wave[count_line], &flux[count_line], 
			 &err[count_line], &bb[count_line], &berr[count_line]);
          if (items == 5)
            ++count_line;
          else
	   {
	   if (items == 2 || items == 3)
	    {
		bb[count_line]=0.;
		if (items == 3) bb[count_line]=err[count_line];
		err[count_line]=sqrt(flux[count_line]);
		berr[count_line]=sqrt(bb[count_line]);
		++count_line;
	    }
	   else
            {
              g_warning("%s: Invalid line: %s", path, line);
              free(wave); free(flux); free(err); free(bb); free(berr);
              return (-1);
            }
	   }
        }

     }
fclose(fd);
  if (float_min(flux,count,&i) < 0.)
    {
      g_error("A background subtracted spectrum is not allowed: %s.\n",path);
      exit(8);
    }

  if (count_line < count)
    {
      g_warning("Race condition detected\n");
      free(wave); free(flux); free(err); free(bb); free(berr);
      return (-1);
    }
  else
    {
      if (verbose > 1)
	g_message("%s: %d datapoints\n", path, count_line);
    }
  for (lines=0;lines<count;++lines) if (bb[lines]>flux[lines]) ++ibgs;

  if (ibgs>count/2.)
    {
      g_warning("More than half of the bins");
      g_warning("contain more counts in the background");
      g_warning("than in the spectrum.");
      g_warning("This might indicate that columns in the spectrum file are mixed up. ");

      fd = fopen ("spectrum.swap", "w");
      if (fd)
	{
	  for (i=0;i<count;++i)
	    fprintf(fd, "%8.3f %4.2f %4.2f %4.2f %4.2f\n",
		    wave[i], bb[i], berr[i], flux[i], err[i]);
	  fclose(fd);
	  g_warning(" <<<< a file 'spectrum.swap' is provided now with swapped columns  >>>>");
	}
    }

  free(berr);

  return (count);
}
